=== modified file 'dev.cfg'
--- dev.cfg	2007-01-14 05:40:40 +0000
+++ dev.cfg	2007-12-12 08:36:51 +0000
@@ -36,6 +36,10 @@
 server.environment="production"
 #autoreload.package="loggerhead"
 
+# We do *not* want our query arguments turned into unicode strings (as
+# we use them for revision/file ids, and these must not be unicode).
+decoding_filter.on = False
+
 # session_filter.on = True
 
 # Set to True if you'd like to abort execution if a controller gets an

=== modified file 'loggerhead/branchview.py'
--- loggerhead/branchview.py	2007-08-23 15:03:26 +0000
+++ loggerhead/branchview.py	2008-02-27 03:36:09 +0000
@@ -53,12 +53,12 @@
         self._config = config
         self._project_config = project_config
         self.log = logging.getLogger('loggerhead.%s' % (name,))
-        
+
         # branch history
         self._history_lock = threading.RLock()
         self._history = None
         self._closed = False
-        
+
         self.changes = ChangeLogUI(self)
         self.revision = RevisionUI(self)
         self.files = InventoryUI(self)
@@ -66,12 +66,12 @@
         self.download = DownloadUI(self)
         self.atom = AtomUI(self)
         self.bundle = BundleUI(self)
-        
+
         # force history object to be loaded:
         self.get_history()
-        
+
         turbogears.startup.call_on_shutdown.append(self.close)
-    
+
     @with_history_lock
     def close(self):
         # it's important that we cleanly detach the history, so the cache
@@ -80,13 +80,13 @@
         self._history.detach()
         self._history = None
         self._closed = True
-            
+
     config = property(lambda self: self._config)
-    
+
     name = property(lambda self: self._name)
 
     group_name = property(lambda self: self._group_name)
-    
+
     def _get_friendly_name(self):
         name = self._config.get('branch_name', None)
         if name is not None:
@@ -106,9 +106,9 @@
         # try branch-specific config?
         description = self.get_history().get_config().get_user_option('description')
         return description
-        
+
     description = property(_get_description)
-    
+
     def _get_branch_url(self):
         url = self._config.get('url', None)
         if url is not None:
@@ -120,9 +120,9 @@
         # try branch-specific config?
         url = self.get_history().get_config().get_user_option('public_branch')
         return url
-        
+
     branch_url = property(_get_branch_url)
-    
+
     @turbogears.expose()
     def index(self):
         raise HTTPRedirect(self.url('/changes'))
@@ -152,13 +152,14 @@
                 _history.use_file_cache(FileChangeCache(_history, cache_path))
                 _history.use_search_index(TextIndex(_history, cache_path))
         return self._history
-    
+
     def check_rebuild(self):
         h = self.get_history()
         if h is not None:
             h.check_rebuild()
-    
+
     def url(self, elements, **kw):
+        "build an url relative to this branch"
         if not isinstance(elements, list):
             elements = [elements]
         if elements[0].startswith('/'):
@@ -166,6 +167,10 @@
         elements = [urllib.quote(x) for x in elements]
         return turbogears.url([ '/' + self.group_name, self.name ] + elements, **kw)
 
+    def context_url(self, elements, **kw):
+        "build an url relative to this branch, bringing along browsing context"
+        return self.url(elements, **util.get_context(**kw))
+
     def last_updated(self):
         h = self.get_history()
         change = h.get_changes([ h.last_revid ])[0]

=== modified file 'loggerhead/changecache.py'
--- loggerhead/changecache.py	2007-09-07 16:41:32 +0000
+++ loggerhead/changecache.py	2008-02-27 03:36:09 +0000
@@ -26,13 +26,10 @@
 """
 
 import cPickle
-import logging
 import os
-import threading
 import time
 
 from loggerhead import util
-from loggerhead.util import decorator
 from loggerhead.lockfile import LockFile
 
 
@@ -107,7 +104,7 @@
     def __init__(self, history, cache_path):
         self.history = history
         self.log = history.log
-        
+
         if not os.path.exists(cache_path):
             os.mkdir(cache_path)
 
@@ -130,7 +127,7 @@
     def close(self):
         self.log.debug('Closing cache file.')
         self._closed = True
-    
+
     @with_lock
     def closed(self):
         return self._closed
@@ -138,7 +135,7 @@
     @with_lock
     def flush(self):
         pass
-    
+
     @with_lock
     def get_changes(self, revid_list):
         """
@@ -191,7 +188,7 @@
         """
         if self.closed() or self.full():
             return
-        
+
         self.log.info('Building revision cache...')
         start_time = time.time()
         last_update = time.time()

=== modified file 'loggerhead/controllers/__init__.py'
--- loggerhead/controllers/__init__.py	2007-06-19 00:27:53 +0000
+++ loggerhead/controllers/__init__.py	2008-02-27 03:35:24 +0000
@@ -20,17 +20,14 @@
 import logging
 import os
 import re
-import sys
-import time
 
 import turbogears
 from turbogears import controllers
-from cherrypy import NotFound
 from configobj import ConfigObj
 
 from loggerhead import util
 from loggerhead.branchview import BranchView
-from loggerhead.history import History, is_branch
+from loggerhead.history import is_branch
 
 log = logging.getLogger("loggerhead.controllers")
 

=== modified file 'loggerhead/controllers/annotate_ui.py'
--- loggerhead/controllers/annotate_ui.py	2007-08-23 12:18:18 +0000
+++ loggerhead/controllers/annotate_ui.py	2008-02-27 03:35:24 +0000
@@ -17,15 +17,13 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-import datetime
 import logging
 import os
 import posixpath
-import textwrap
 import time
 
 import turbogears
-from cherrypy import HTTPError, InternalError, session
+from cherrypy import HTTPError
 
 from loggerhead import util
 

=== modified file 'loggerhead/controllers/atom_ui.py'
--- loggerhead/controllers/atom_ui.py	2007-08-30 11:39:10 +0000
+++ loggerhead/controllers/atom_ui.py	2008-02-27 03:35:24 +0000
@@ -17,7 +17,6 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-import cherrypy
 import turbogears
 
 from loggerhead import util

=== modified file 'loggerhead/controllers/bundle_ui.py'
--- loggerhead/controllers/bundle_ui.py	2007-08-23 12:18:18 +0000
+++ loggerhead/controllers/bundle_ui.py	2008-02-27 03:35:24 +0000
@@ -16,14 +16,11 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-import os
 import time
 
 import turbogears
 from cherrypy import HTTPRedirect, InternalError, response
 
-from loggerhead import util
-
 
 class BundleUI (object):
 
@@ -54,7 +51,7 @@
                 self.log.exception('Exception fetching bundle')
                 raise InternalError('Could not fetch bundle')
 
-            response.headers['Content-Type'] = 'text/plain'
+            response.headers['Content-Type'] = 'application/octet-stream'
             response.headers['Content-Length'] = len(bundle_data)
             response.body = bundle_data
             self.log.info('/bundle: %r seconds' % (time.time() - z,))

=== modified file 'loggerhead/controllers/changelog_ui.py'
--- loggerhead/controllers/changelog_ui.py	2007-08-23 12:18:18 +0000
+++ loggerhead/controllers/changelog_ui.py	2008-02-27 03:35:24 +0000
@@ -17,11 +17,10 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-import os
 import time
 
 import turbogears
-from cherrypy import InternalError, session
+from cherrypy import InternalError
 
 from loggerhead import util
 
@@ -70,9 +69,9 @@
                     else:
                         i = None
                     scan_list = revid_list[i:]
-                entry_list = scan_list[:pagesize]
-                entries = h.get_changes(entry_list)
-                h.add_changes(entries)
+                change_list = scan_list[:pagesize]
+                changes = list(h.get_changes(change_list))
+                h.add_changes(changes)
             except:
                 self.log.exception('Exception fetching changes')
                 raise InternalError('Could not fetch changes')
@@ -83,25 +82,24 @@
                 navigation.query = query
             util.fill_in_navigation(navigation)
 
-            entries = list(entries)
             # add parent & merge-point branch-nick info, in case it's useful
-            h.get_branch_nicks(entries)
+            h.get_branch_nicks(changes)
 
             # does every change on this page have the same committer?  if so,
             # tell the template to show committer info in the "details block"
             # instead of on each line.
             all_same_author = True
 
-            if entries:
-                author = entries[0].author
-                for e in entries[1:]:
+            if changes:
+                author = changes[0].author
+                for e in changes[1:]:
                     if e.author != author:
                         all_same_author = False
                         break
 
             vals = {
                 'branch': self._branch,
-                'changes': entries,
+                'changes': changes,
                 'util': util,
                 'history': h,
                 'revid': revid,

=== modified file 'loggerhead/controllers/download_ui.py'
--- loggerhead/controllers/download_ui.py	2007-08-23 12:18:18 +0000
+++ loggerhead/controllers/download_ui.py	2008-02-27 03:35:24 +0000
@@ -17,18 +17,12 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-import datetime
 import logging
 import mimetypes
-import os
-import posixpath
-import textwrap
 import time
 
 import turbogears
-from cherrypy import HTTPRedirect, response, session
-
-from loggerhead import util
+from cherrypy import HTTPRedirect, response
 
 
 log = logging.getLogger("loggerhead.controllers")

=== modified file 'loggerhead/controllers/inventory_ui.py'
--- loggerhead/controllers/inventory_ui.py	2007-08-22 22:39:43 +0000
+++ loggerhead/controllers/inventory_ui.py	2008-02-27 03:35:24 +0000
@@ -17,15 +17,12 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-import datetime
 import logging
-import os
 import posixpath
-import textwrap
 import time
 
 import turbogears
-from cherrypy import InternalError, session
+from cherrypy import InternalError
 
 from loggerhead import util
 

=== modified file 'loggerhead/controllers/revision_ui.py'
--- loggerhead/controllers/revision_ui.py	2007-08-23 12:18:18 +0000
+++ loggerhead/controllers/revision_ui.py	2008-02-27 03:35:24 +0000
@@ -17,14 +17,10 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-import datetime
-import logging
-import os
-import textwrap
 import time
 
 import turbogears
-from cherrypy import InternalError, session
+from cherrypy import InternalError
 
 from loggerhead import util
 

=== modified file 'loggerhead/daemon.py'
--- loggerhead/daemon.py	2007-06-18 23:07:53 +0000
+++ loggerhead/daemon.py	2008-02-27 03:36:09 +0000
@@ -1,7 +1,6 @@
 # daemon code from ASPN
 
 import os
-import sys
 
 
 def daemonize(pidfile, home):
@@ -20,41 +19,41 @@
     except OSError, e:
         raise Exception("%s [%d]" % (e.strerror, e.errno))
 
-    if (pid == 0):	# The first child.
+    if (pid == 0):      # The first child.
         os.setsid()
 
         try:
-            pid = os.fork()	# Fork a second child.
+            pid = os.fork()     # Fork a second child.
         except OSError, e:
             raise Exception, "%s [%d]" % (e.strerror, e.errno)
 
-        if (pid == 0):	# The second child.
+        if (pid == 0):  # The second child.
             os.chdir(WORKDIR)
             os.umask(UMASK)
         else:
-            os._exit(0)	# Exit parent (the first child) of the second child.
+            os._exit(0) # Exit parent (the first child) of the second child.
     else:
-        os._exit(0)	# Exit parent of the first child.
+        os._exit(0)     # Exit parent of the first child.
 
-    #import resource		# Resource usage information.
+    #import resource            # Resource usage information.
     #maxfd = resource.getrlimit(resource.RLIMIT_NOFILE)[1]
     #if (maxfd == resource.RLIM_INFINITY):
     #    maxfd = MAXFD
-  
+
     # Iterate through and close all file descriptors.
     #for fd in range(3, maxfd):
     #    try:
     #        os.close(fd)
-    #    except OSError:	# ERROR, fd wasn't open to begin with (ignored)
+    #    except OSError:        # ERROR, fd wasn't open to begin with (ignored)
     #        pass
 
     # This call to open is guaranteed to return the lowest file descriptor,
     # which will be 0 (stdin), since it was closed above.
-    #os.open(REDIRECT_TO, os.O_RDWR)	# standard input (0)
+    #os.open(REDIRECT_TO, os.O_RDWR)    # standard input (0)
 
     # Duplicate standard input to standard output and standard error.
-    #os.dup2(0, 1)			# standard output (1)
-    #os.dup2(0, 2)			# standard error (2)
+    #os.dup2(0, 1)                      # standard output (1)
+    #os.dup2(0, 2)                      # standard error (2)
 
     f = open(pidfile, 'w')
     f.write('%d\n' % (os.getpid(),))

=== modified file 'loggerhead/history.py'
--- loggerhead/history.py	2007-09-10 13:41:13 +0000
+++ loggerhead/history.py	2008-02-27 03:36:09 +0000
@@ -27,14 +27,9 @@
 
 
 import bisect
-import cgi
 import datetime
 import logging
-import os
-import posixpath
 import re
-import shelve
-import sys
 import textwrap
 import threading
 import time
@@ -44,15 +39,12 @@
 from loggerhead.util import decorator
 
 import bzrlib
-import bzrlib.annotate
 import bzrlib.branch
 import bzrlib.bundle.serializer
-import bzrlib.decorators
 import bzrlib.diff
 import bzrlib.errors
 import bzrlib.progress
 import bzrlib.revision
-import bzrlib.textfile
 import bzrlib.tsort
 import bzrlib.ui
 
@@ -60,6 +52,19 @@
 with_branch_lock = util.with_lock('_lock', 'branch')
 
 
+@decorator
+def with_bzrlib_read_lock(unbound):
+    def bzrlib_read_locked(self, *args, **kw):
+        #self.log.debug('-> %r bzr lock', id(threading.currentThread()))
+        self._branch.repository.lock_read()
+        try:
+            return unbound(self, *args, **kw)
+        finally:
+            self._branch.repository.unlock()
+            #self.log.debug('<- %r bzr lock', id(threading.currentThread()))
+    return bzrlib_read_locked
+
+
 # bzrlib's UIFactory is not thread-safe
 uihack = threading.local()
 
@@ -197,10 +202,7 @@
         self = cls()
         self._branch = branch
         self._last_revid = self._branch.last_revision()
-        if self._last_revid is not None:
-            self._revision_graph = branch.repository.get_revision_graph(self._last_revid)
-        else:
-            self._revision_graph = {}
+        self._revision_graph = branch.repository.get_revision_graph(self._last_revid)
 
         if name is None:
             name = self._branch.nick
@@ -210,17 +212,20 @@
         self._full_history = []
         self._revision_info = {}
         self._revno_revid = {}
-        self._merge_sort = bzrlib.tsort.merge_sort(self._revision_graph, self._last_revid, generate_revno=True)
+        if bzrlib.revision.is_null(self._last_revid):
+            self._merge_sort = []
+        else:
+            self._merge_sort = bzrlib.tsort.merge_sort(
+                self._revision_graph, self._last_revid, generate_revno=True)
         for (seq, revid, merge_depth, revno, end_of_merge) in self._merge_sort:
             self._full_history.append(revid)
             revno_str = '.'.join(str(n) for n in revno)
             self._revno_revid[revno_str] = revid
             self._revision_info[revid] = (seq, revid, merge_depth, revno_str, end_of_merge)
-
         # cache merge info
         self._where_merged = {}
         for revid in self._revision_graph.keys():
-            if not revid in self._full_history:
+            if self._revision_info[revid][2] == 0:
                 continue
             for parent in self._revision_graph[revid]:
                 self._where_merged.setdefault(parent, set()).add(revid)
@@ -231,13 +236,21 @@
     @classmethod
     def from_folder(cls, path, name=None):
         b = bzrlib.branch.Branch.open(path)
-        return cls.from_branch(b, name)
+        b.lock_read()
+        try:
+            return cls.from_branch(b, name)
+        finally:
+            b.unlock()
 
     @with_branch_lock
     def out_of_date(self):
         # the branch may have been upgraded on disk, in which case we're stale.
+        newly_opened = bzrlib.branch.Branch.open(self._branch.base)
         if self._branch.__class__ is not \
-               bzrlib.branch.Branch.open(self._branch.base).__class__:
+               newly_opened.__class__:
+            return True
+        if self._branch.repository.__class__ is not \
+               newly_opened.repository.__class__:
             return True
         return self._branch.last_revision() != self._last_revid
 
@@ -250,6 +263,10 @@
     def use_search_index(self, index):
         self._index = index
 
+    @property
+    def has_revisions(self):
+        return not bzrlib.revision.is_null(self.last_revid)
+
     @with_branch_lock
     def detach(self):
         # called when a new history object needs to be created, because the
@@ -449,14 +466,16 @@
         determine the revision list we're viewing (start_revid, file_id, query)
         and where we are in it (revid).
 
-        if a query is given, we're viewing query results.
-        if a file_id is given, we're viewing revisions for a specific file.
-        if a start_revid is given, we're viewing the branch from a
-            specific revision up the tree.
-        (these may be combined to view revisions for a specific file, from
-            a specific revision, with a specific search query.)
-
-        returns a new (revid, start_revid, revid_list, scan_list) where:
+            - if a query is given, we're viewing query results.
+            - if a file_id is given, we're viewing revisions for a specific
+              file.
+            - if a start_revid is given, we're viewing the branch from a
+              specific revision up the tree.
+
+        these may be combined to view revisions for a specific file, from
+        a specific revision, with a specific search query.
+
+        returns a new (revid, start_revid, revid_list) where:
 
             - revid: current position within the view
             - start_revid: starting revision of this view
@@ -589,6 +608,10 @@
 
     @with_branch_lock
     def get_changes(self, revid_list):
+        """Return a list of changes objects for the given revids.
+
+        Revisions not present and NULL_REVISION will be ignored.
+        """
         if self._change_cache is None:
             changes = self.get_changes_uncached(revid_list)
         else:
@@ -601,6 +624,12 @@
         for change in changes:
             merge_revids = self.simplify_merge_point_list(self.get_merge_point_list(change.revid))
             change.merge_points = [util.Container(revid=r, revno=self.get_revno(r)) for r in merge_revids]
+            if len(change.parents) > 0:
+                if isinstance(change.parents[0], util.Container):
+                    # old cache stored a potentially-bogus revno
+                    change.parents = [util.Container(revid=p.revid, revno=self.get_revno(p.revid)) for p in change.parents]
+                else:
+                    change.parents = [util.Container(revid=r, revno=self.get_revno(r)) for r in change.parents]
             change.revno = self.get_revno(change.revid)
 
         parity = 0
@@ -610,46 +639,62 @@
 
         return changes
 
-    # alright, let's profile this sucka.
-    def _get_changes_profiled(self, revid_list, get_diffs=False):
+    # alright, let's profile this sucka. (FIXME remove this eventually...)
+    def _get_changes_profiled(self, revid_list):
         from loggerhead.lsprof import profile
         import cPickle
-        ret, stats = profile(self.get_changes_uncached, revid_list, get_diffs)
+        ret, stats = profile(self.get_changes_uncached, revid_list)
         stats.sort()
         stats.freeze()
         cPickle.dump(stats, open('lsprof.stats', 'w'), 2)
         self.log.info('lsprof complete!')
         return ret
 
-    def _get_deltas_for_revisions_with_trees(self, entries):
-        """Produce a generator of revision deltas.
+    @with_branch_lock
+    @with_bzrlib_read_lock
+    def get_changes_uncached(self, revid_list):
+        revid_list = filter(lambda revid: not bzrlib.revision.is_null(revid),
+                            revid_list)
+        repo = self._branch.repository
+        rev_list = repo.get_revisions(
+            repo.get_graph().get_parent_map(revid_list))
+
+        return [self._change_from_revision(rev) for rev in rev_list]
+
+    def _get_deltas_for_revisions_with_trees(self, revisions):
+        """Produce a list of revision deltas.
 
         Note that the input is a sequence of REVISIONS, not revision_ids.
         Trees will be held in memory until the generator exits.
         Each delta is relative to the revision's lefthand predecessor.
+        (This is copied from bzrlib.)
         """
         required_trees = set()
-        for entry in entries:
-            required_trees.add(entry.revid)
-            required_trees.update([p.revid for p in entry.parents[:1]])
+        for revision in revisions:
+            required_trees.add(revision.revid)
+            required_trees.update([p.revid for p in revision.parents[:1]])
         trees = dict((t.get_revision_id(), t) for
                      t in self._branch.repository.revision_trees(required_trees))
         ret = []
         self._branch.repository.lock_read()
         try:
-            for entry in entries:
-                if not entry.parents:
+            for revision in revisions:
+                if not revision.parents:
                     old_tree = self._branch.repository.revision_tree(
                         bzrlib.revision.NULL_REVISION)
                 else:
-                    old_tree = trees[entry.parents[0].revid]
-                tree = trees[entry.revid]
+                    old_tree = trees[revision.parents[0].revid]
+                tree = trees[revision.revid]
                 ret.append(tree.changes_from(old_tree))
             return ret
         finally:
             self._branch.repository.unlock()
 
-    def entry_from_revision(self, revision):
+    def _change_from_revision(self, revision):
+        """
+        Given a bzrlib Revision, return a processed "change" for use in
+        templates.
+        """
         commit_time = datetime.datetime.fromtimestamp(revision.timestamp)
 
         parents = [util.Container(revid=r, revno=self.get_revno(r)) for r in revision.parent_ids]
@@ -664,31 +709,10 @@
             'short_comment': short_message,
             'comment': revision.message,
             'comment_clean': [util.html_clean(s) for s in message],
-            'parents': parents,
+            'parents': revision.parent_ids,
         }
         return util.Container(entry)
 
-    @with_branch_lock
-    def get_changes_uncached(self, revid_list):
-        # Because we may loop and call get_revisions multiple times (to throw
-        # out dud revids), we grab a read lock.
-        self._branch.lock_read()
-        try:
-            while True:
-                try:
-                    rev_list = self._branch.repository.get_revisions(revid_list)
-                except (KeyError, bzrlib.errors.NoSuchRevision), e:
-                    # this sometimes happens with arch-converted branches.
-                    # i don't know why. :(
-                    self.log.debug('No such revision (skipping): %s', e)
-                    revid_list.remove(e.revision)
-                else:
-                    break
-
-            return [self.entry_from_revision(rev) for rev in rev_list]
-        finally:
-            self._branch.unlock()
-
     def get_file_changes_uncached(self, entries):
         delta_list = self._get_deltas_for_revisions_with_trees(entries)
 
@@ -709,11 +733,11 @@
 
     @with_branch_lock
     def get_change_with_diff(self, revid, compare_revid=None):
-        entry = self.get_changes([revid])[0]
+        change = self.get_changes([revid])[0]
 
         if compare_revid is None:
-            if entry.parents:
-                compare_revid = entry.parents[0].revid
+            if change.parents:
+                compare_revid = change.parents[0].revid
             else:
                 compare_revid = 'null:'
 
@@ -721,11 +745,10 @@
         rev_tree2 = self._branch.repository.revision_tree(revid)
         delta = rev_tree2.changes_from(rev_tree1)
 
-        entry.changes = self.parse_delta(delta)
-
-        entry.changes.modified = self._parse_diffs(rev_tree1, rev_tree2, delta)
-
-        return entry
+        change.changes = self.parse_delta(delta)
+        change.changes.modified = self._parse_diffs(rev_tree1, rev_tree2, delta)
+
+        return change
 
     @with_branch_lock
     def get_file(self, file_id, revid):
@@ -966,4 +989,3 @@
         s = StringIO()
         bzrlib.bundle.serializer.write_bundle(self._branch.repository, revid, compare_revid, s)
         return s.getvalue()
-

=== removed file 'loggerhead/history2.py'
--- loggerhead/history2.py	2007-06-25 10:16:34 +0000
+++ loggerhead/history2.py	1970-01-01 00:00:00 +0000
@@ -1,194 +0,0 @@
-"""Extract data suitable for web presentation from Bazaar branches.
-
-The module is the interface between Loggerhead and bzrlib.  Other code in
-Loggerhead should not use bzrlib APIs, and functions and methods in this
-module should not return bzrlib data types.
-
-The core method is `History.getRevisionInfo`.  This method is meant to be
-cheap, so you should not worry about calling it multiple times with the same
-argument or calling it just to retreive a subset of the information it
-returns.
-
-Revisions are described using good old bzrlib revision ids.
-
-Revision ids, file ids, file names and file paths are always utf-8 encoded 8
-bit strings.
-"""
-
-class FileDelta(object):
-    """Information about how an inventory entry changed in a revision.
-
-    All attribute names are fairly self explanatory.  The ``*_parent`` fields
-    are the file id of the parent or ``None`` for the root.
-
-    :ivar file_id:
-    :ivar old_path:
-    :ivar new_path:
-    :ivar old_name:
-    :ivar new_name:
-    :ivar old_parent:
-    :ivar new_parent:
-    :ivar old_kind:
-    :ivar new_kind:
-    :ivar content_change:
-    :type content_change: ``bool``
-    :ivar execute_change: A boolean reflecting the new state if it changed, or
-                          ``None`` if it did not.
-    :type execute_change: ``bool`` or ``None``
-    """
-
-
-class FileChanges(object):
-    """Contains information about which files changed in a revision.
-
-    Note that the 'files changed' information is relative to left-most parent
-    by default, as in 'bzr log -v'.
-
-    :ivar deltas: A list of `FileDelta` objects.
-    :ivar fileset: ``set(d.file_id for d in self.deltas)``, but cached.
-    """
-
-class RevisionInfo(object):
-    """Contains all the immutable information Loggerhead needs about a
-    particular revision.
-
-    :ivar revid: Obvious.
-    :ivar date: The date and time this revision was committed.
-    :type date: ``datetime.datetime``
-    :ivar committer: The committer.
-    :type committer: utf-8 encoded ``str``
-    :ivar revprops: The branch properties.
-    :type revprops: A dictionary mapping ``str``\ s to ``str``\ s (both utf-8
-                    encoded).
-    :ivar message: The commit message of this revision.
-    :type message: utf-8 encoded ``str``
-    :ivar parents: The list of the revids of the revision's parents.
-    """
-
-class BranchRevisionInfo(object):
-    """Contains all the mutable information Loggerhead needs about a
-    particular revision, and points to the immutable data.
-
-    :ivar revno: The dotted revno of this revision in this branch.
-    :type revno: ``str``
-    :ivar where_merged: The list of revids that have this revision as a
-                        parent.
-    :ivar info: The corresponding `RevisionInfo`.
-    """
-
-class FileEntry(object):
-    """An entry in the list returned by `getFileList`.
-
-    :ivar path: The path of the object in the revision passed to
-                `getFileList`.
-    :ivar fileid: Obvious.
-    :ivar kind: one of ``'file'``, ``'executable'``, ``'link'`` or
-                ``'directory'``.
-    :type kind: ``str``
-    :ivar last_changed_revid: The revid of the revision in which this path
-                              last changed.  Not that this is recursive,
-                              i.e. the last_changed_revid for a directory is
-                              the revid in which the directory or anything
-                              contained in in changed.x
-    :ivar size: The size of this object in this revision in bytes.
-    :type size: ``int``
-    """
-
-class History(object):
-    """Provide the information loggerhead needs about a bzrlib Branch.
-
-    An instance of this class is effectively a wrapper around a
-    `bzrlib.branch.Branch` that translates the information provided by bzrlib
-    into a form convenient for use in loggerhead.
-
-    :ivar last_revision: The revid of the tip of the branch.
-    """
-
-    @classmethod
-    def fromBranch(cls, branch):
-        """Create and initialize a `History` object from a
-        `bzrlib.branch.Branch`.
-        """
-
-    def outOfDate(self):
-        """Decide whether this History object is still current.
-
-        At least to start with, an out of date History object should be thrown
-        away and a fresh one created.
-        """
-
-    def getRevisionInfo(self, revid):
-        """Find the `BranchRevisionInfo` object for a given revision id.
-
-        This operation is to be thought of as 'cheap', which is to say that
-        you should not worry about calling it repeatedly with the same
-        argument or about calling it just to retrieve a subset of the
-        information it includes.
-
-        :param revid: a revision id.
-        :returns: a `BranchRevisionInfo` object or ``None`` if the given
-                  revision id does not exist in the branch.
-        """
-
-    def getChanges(self, revid, other_revid=None):
-        """Find the `FileChanges` object between two revision.
-
-        :param revid: A revision id.
-        :param other_revid: The revision id to compare against.  If ``None``
-                            or not given, compare against the leftmost parent
-                            of ``revid``.
-        :returns: The corresponding `FileChanges` object.
-        """
-
-    # There will need to be some more methods here to allow searching for
-    # revisions by date, committer, commit message, etc...
-
-    def getDiff(self, revidA, revidB):
-        """Compute the diff between two revisions.
-
-        Not sure what this will return yet, probably something similar to but
-        more structured than the 'chunks' loggerhead already works with.
-        """
-
-    def getDirList(self, fileid, revid):
-        """List the files in the given directory as of the given revision.
-
-        :return: a list of `FileEntry` objects.
-        """
-
-    def getFileAnnotation(self, fileid, revid):
-        """Annotate the given file.
-
-        Not sure what this should return, mostly likely a list of
-        (revid-or-None, line contents) (and ``None`` for a binary file).
-        """
-
-    def getFileText(self, fileid, revid):
-        """Fetch the contents of a file at a particular revision.
-
-        :returns: (filename, data)
-        :rtype: ``(str, str)``
-        """
-
-    def getBundle(self, revid, compare_revid=None):
-        """Return a bundle for this revision.
-
-        Relative to compare_revid, or left-most parent if this is not supplied
-        or None.
-
-        :return: A bundle (in a string).
-        """
-
-    def normalizeRevivionSpec(self, revspec):
-        """Convert a revid or revno or 'head' to a revid.
-
-        :returns: a revision id.
-        """
-
-    def normalizeFileArguments(self, revid, fileid, filepath):
-        """Handle various ways of specifying a file in the branch.
-
-        One of ``fileid`` and ``filepath`` should be None.
-
-        :returns: (fileid, filepath)
-        """

=== removed file 'loggerhead/json.py'
--- loggerhead/json.py	2006-12-11 06:44:19 +0000
+++ loggerhead/json.py	1970-01-01 00:00:00 +0000
@@ -1,12 +0,0 @@
-# This module provides helper functions for the JSON part of your
-# view, if you are providing a JSON-based API for your app.
-
-# Here's what most rules would look like:
-# @jsonify.when("isinstance(obj, YourClass)")
-# def jsonify_yourclass(obj):
-#     return [obj.val1, obj.val2]
-# The goal is to break your objects down into simple values:
-# lists, dicts, numbers and strings
-
-from turbojson.jsonify import jsonify
-

=== modified file 'loggerhead/lockfile.py'
--- loggerhead/lockfile.py	2007-01-14 01:20:39 +0000
+++ loggerhead/lockfile.py	2008-02-27 03:36:09 +0000
@@ -35,7 +35,7 @@
     lock may be acquired multiple times by the same thread, as long as it's
     released an equal number of times.  unlike threading.Lock, this lock can
     be used across processes.
-    
+
     this uses os.open(O_CREAT|O_EXCL), which apparently works even on windows,
     but will not work over NFS, if anyone still uses that.  so don't put the
     cache folder on an NFS server...
@@ -50,7 +50,7 @@
             # remove stale locks left over from a previous run
             if time.time() - os.stat(filename).st_mtime > MAX_STALE_TIME:
                 os.remove(filename)
-    
+
     @with_lock
     def _try_acquire(self):
         if self._count > 0:
@@ -63,7 +63,7 @@
             return True
         except OSError:
             return False
-    
+
     def acquire(self):
         # try over and over, sleeping on exponential backoff with an upper limit of about 5 seconds
         pause = 0.1

=== modified file 'loggerhead/lsprof.py'
--- loggerhead/lsprof.py	2006-12-15 01:14:05 +0000
+++ loggerhead/lsprof.py	2008-02-27 03:36:09 +0000
@@ -36,7 +36,7 @@
         for pp in _g_threadmap.values():
             pp.disable()
         threading.setprofile(None)
-    
+
     threads = {}
     for tid, pp in _g_threadmap.items():
         threads[tid] = Stats(pp.getstats(), {})
@@ -112,7 +112,7 @@
         self.out_file = None
 
     def output(self, out_file):
-        self.out_file = out_file        
+        self.out_file = out_file
         print >> out_file, 'events: Ticks'
         self._print_summary()
         for entry in self.data:

=== removed file 'loggerhead/model.py'
--- loggerhead/model.py	2006-12-11 06:44:19 +0000
+++ loggerhead/model.py	1970-01-01 00:00:00 +0000
@@ -1,10 +0,0 @@
-from sqlobject import *
-
-from turbogears.database import PackageHub
-
-hub = PackageHub("loggerhead")
-__connection__ = hub
-
-# class YourDataClass(SQLObject):
-#     pass
-

=== modified file 'loggerhead/static/css/style.css'
--- loggerhead/static/css/style.css	2007-07-04 12:08:51 +0000
+++ loggerhead/static/css/style.css	2007-09-04 01:47:51 +0000
@@ -216,7 +216,7 @@
 }
 
 .log-entries .revision-header {
-    font-size: normal;
+    font-size: 1.0em;
 }
 
 .log-entries .revision-header td {
@@ -315,10 +315,6 @@
     padding-bottom: 2px;
 }
 
-.revision-info table td.date {
-    font-family: monospace;
-}
-
 .revision-info table td.description {
     font-family: monospace;
 }
@@ -503,6 +499,8 @@
     font-family: monospace;
     font-size: smaller;
     width: 5em;
+    line-height: 1em;
+    vertical-align: bottom;
 }
 
 .inventory td.revision {
@@ -513,7 +511,7 @@
 
 .inventory td.date {
     padding-right: 1em;
-    width: 10em;
+    width: 11em;
 }
 
 .inventory td.changes-link {

=== modified file 'loggerhead/templates/annotate.kid'
--- loggerhead/templates/annotate.kid	2007-06-21 16:39:33 +0000
+++ loggerhead/templates/annotate.kid	2007-09-04 17:06:03 +0000
@@ -31,7 +31,7 @@
             <td class="lineno ${line.status}"><a name="L${line.lineno}"> ${line.lineno} </a></td>
             <td class="revno ${line.status}">
                 <a py:if="line.status=='changed'" href="${branch.url('/revision', **util.get_context(clear=1, start_revid=line.change.revid, file_id=file_id))}"
-                    title="${line.change.revno} by ${util.hide_email(line.change.author)}, on ${line.change.date.strftime('%d %b %Y %H:%M')} (${util.ago(line.change.date)})">${util.trunc(line.change.revno)}</a>
+                    title="${line.change.revno} by ${util.hide_email(line.change.author)}, on ${util.format_date(line.change.date)} (${util.ago(line.change.date)})">${util.trunc(line.change.revno)}</a>
             </td>
             <td class="text ${line.status}"> ${line.text} </td>
         </tr>

=== modified file 'loggerhead/templates/browse.kid'
--- loggerhead/templates/browse.kid	2007-06-08 11:52:33 +0000
+++ loggerhead/templates/browse.kid	2008-02-27 01:55:39 +0000
@@ -34,7 +34,7 @@
             <tr>
             <td class="name"> <a href="${tg.url([ '/' + project.name, view.name ])}">${view.friendly_name}</a> </td>
             <td class="description"> ${view.description} </td>
-            <td class="last-update"> ${view.get_history().last_revid and util.ago(view.last_updated()) or "(no revisions)"} </td>
+            <td class="last-update"> ${view.get_history().has_revisions and util.ago(view.last_updated()) or "(no revisions)"} </td>
             <td class="feed">
                 <a href="${tg.url([ '/' + project.name, view.name, 'atom' ])}" title="atom feed">
                     <img src="${tg.url('/static/images/feed-icon-16x16.gif')}" alt="atom feed" />

=== modified file 'loggerhead/templates/changelog.kid'
--- loggerhead/templates/changelog.kid	2007-06-26 05:34:28 +0000
+++ loggerhead/templates/changelog.kid	2007-09-04 17:21:59 +0000
@@ -14,7 +14,22 @@
         <a href="${branch.url([ '/annotate', revid ], **util.get_context(file_id=file_id))}" title="Annotate ${filename}">${filename}</a>
     </span>
     
+    <span py:def="file_diff_link(revid, filename)">
+        <a href="javascript:diff_url('${branch.context_url([ '/revision', revid ]) + '#' + filename}')"
+           title="View diff for ${filename}">${filename}</a>
+    </span>
+    
     ${use_collapse_buttons()}
+
+    <script type="text/javascript"> <!--
+    function diff_url(url) {
+        if (document.cookie.indexOf('diff=unified') >= 0) {
+            this.location.href = url + "-u";
+        } else {
+            this.location.href = url + "-s";
+        }
+    }
+    // --> </script>
 </head>
 
 <body onload="javascript:sortCollapseElements();">
@@ -62,7 +77,7 @@
             <span py:strip="True" py:if="not all_same_author">
                 <td class="author"> ${util.trunc(util.hide_email(entry.author), 20)} </td>
             </span>
-            <td class="date"> ${entry.date.strftime('%Y-%m-%d, %H:%M')} &nbsp; (${util.ago(entry.date)}) </td>
+            <td class="date"> ${util.format_date(entry.date)} &nbsp; (${util.ago(entry.date)}) </td>
             <td class="inventory-link"> 
                 <a href="${branch.url([ '/files', entry.revid ])}"
                     title="Files at revision ${entry.revno}"> files</a>
@@ -83,9 +98,9 @@
                 <tr py:if="len(entry.parents) > 1">
                     <th class="parents"> merged from: </th>
                     <td class="parents">
-                        <span py:for="parent in entry.parents"><span py:if="parent.revid != entry.parents[0].revid">
-                            ${loglink(parent.revid, '(' + parent.revno + util.if_present(' %s', parent.branch_nick) + ')')} <br />
-                        </span></span>
+                        <span py:for="parent in entry.parents[1:]">
+                            ${loglink(parent.revid, parent.revno + util.if_present(' (%s)', parent.branch_nick))} <br />
+                        </span>
                     </td>
                 </tr>
                 <span py:strip="True" py:if="all_same_author">
@@ -115,8 +130,7 @@
                 <tr py:if="entry.changes.modified">
                     <th class="files"> files modified: </th>
                     <td class="files"> <span py:for="item in entry.changes.modified">
-                        <span class="filename">${file_link(item.filename, item.file_id, entry.revid)}</span> &nbsp;
-                        <a href="${branch.url([ '/revision', entry.revid ], **util.get_context()) + '#' + item.filename}" class="jump">&#8594; diff</a>
+                        <span class="filename">${file_diff_link(entry.revid, item.filename)}</span>
                         <br />
                     </span> </td>
                 </tr>

=== modified file 'loggerhead/templates/inventory.kid'
--- loggerhead/templates/inventory.kid	2007-06-25 06:44:52 +0000
+++ loggerhead/templates/inventory.kid	2007-09-04 17:06:03 +0000
@@ -12,9 +12,9 @@
 
 <h1> <span class="branch-name">${branch.friendly_name}</span> : files for revision ${change.revno}
 	<div class="links">
-	    <div> <b>&#8594;</b> <a href="${branch.url('/revision', **util.get_context(clear=1, start_revid=revid))}">
+	    <div> <b>&#8594;</b> <a href="${branch.context_url('/revision', clear=1, start_revid=revid)}">
 	        view revision </a> </div>
-	    <div> <b>&#8594;</b> <a href="${branch.url('/changes', **util.get_context(clear=1, start_revid=revid))}">
+	    <div> <b>&#8594;</b> <a href="${branch.context_url('/changes', clear=1, start_revid=revid)}">
 	        view branch changes </a> </div>
 	</div>
 </h1>
@@ -28,7 +28,7 @@
         </tr>
         <tr>
             <th class="date">date:</th>
-            <td class="date"> ${change.date.strftime('%d %b %Y %H:%M')} </td>
+            <td class="date"> ${util.format_date(change.date)} </td>
         </tr>
 
         <tr py:if="len(change.merge_points) > 0">
@@ -62,35 +62,34 @@
 <table class="inventory" width="100%">
     <tr class="header">
         <th class="permissions"> Permissions </th>
-        <th> <a href="${branch.url([ '/files', revid ], **util.get_context(sort='filename'))}">Filename</a> </th>
+        <th> <a href="${branch.context_url([ '/files', revid ], sort='filename')}">Filename</a> </th>
         <th> Latest Rev </th>
-        <th> <a href="${branch.url([ '/files', revid ], **util.get_context(sort='date'))}">Last Changed</a> </th>
-        <th> <a href="${branch.url([ '/files', revid ], **util.get_context(sort='size'))}">Size</a> </th>
+        <th> <a href="${branch.context_url([ '/files', revid ], sort='date')}">Last Changed</a> </th>
+        <th> <a href="${branch.context_url([ '/files', revid ], sort='size')}">Size</a> </th>
         <th>  </th>
         <th>  </th>
     </tr>
     
     <tr class="parity1" py:if="updir">
         <td class="permissions">drwxr-xr-x</td>
-        <td class="filename directory"><a href="${branch.url([ '/files', revid ],
-            **util.get_context(file_id=updir_file_id))}"> (up) </a></td>
+        <td class="filename directory"><a href="${branch.context_url([ '/files', revid ], file_id=updir_file_id)}"> (up) </a></td>
         <td> </td> <td> </td> <td> </td> <td> </td>
     </tr>
 
     <tr py:for="file in filelist" class="parity${file.parity}">
         <td class="permissions"> ${util.fake_permissions(file.kind, file.executable)} </td>
         <td class="filename ${file.kind}">
-            <a py:if="file.kind=='directory'" href="${branch.url([ '/files', revid ],
-                **util.get_context(file_id=file.file_id))}">${file.filename}/</a>
+            <a py:if="file.kind=='directory'" href="${branch.context_url([ '/files', revid ],
+                file_id=file.file_id)}">${file.filename}/</a>
             <span py:if="file.kind=='symlink'">${file.filename}@</span>
-            <a py:if="file.kind=='file'" href="${branch.url([ '/annotate', revid ],
-                **util.get_context(file_id=file.file_id))}" title="Annotate ${file.filename}">${file.filename}</a>
+            <a py:if="file.kind=='file'" href="${branch.context_url([ '/annotate', revid ],
+                file_id=file.file_id)}" title="Annotate ${file.filename}">${file.filename}</a>
         </td>
         <td class="revision"> ${revision_link(file.revid, util.trunc(file.change.revno, 15), **util.get_context(start_revid=file.revid, file_id=file.file_id))} </td>
-        <td class="date"> ${file.change.date.strftime('%Y-%m-%d, %H:%M')} </td>
+        <td class="date"> ${util.format_date(file.change.date)} </td>
         <td class="size"> <span py:if="file.kind=='file'"> ${util.human_size(file.size)} </span></td>
         <td class="changes-link"> 
-            <a href="${branch.url('/changes', **util.get_context(start_revid=file.revid, file_id=file.file_id))}"
+            <a href="${branch.context_url('/changes', start_revid=file.revid, file_id=file.file_id)}"
                title="Changes affecting ${file.filename} up to revision ${file.change.revno}"> changes </a>
         </td>
         <td class="download-link">

=== modified file 'loggerhead/templates/master.kid'
--- loggerhead/templates/master.kid	2007-01-24 19:17:40 +0000
+++ loggerhead/templates/master.kid	2007-09-02 22:54:43 +0000
@@ -19,8 +19,8 @@
               file_id=getattr(navigation, 'file_id', None))}"><table><tr>
             <td><span class="buttons">
                 <!-- ! navbar buttons never change, from now on.  i decree it! -->
-                <a href="${branch.url('/changes', **util.get_context(clear=1))}"> changes </a>
-                <a href="${branch.url('/files', **util.get_context(clear=1))}"> files </a>
+                <a href="${branch.context_url('/changes', clear=1)}"> changes </a>
+                <a href="${branch.context_url('/files', clear=1)}"> files </a>
                 <span class="search"> search: <input type="text" name="q" /> </span>
             </span></td>
             <td align="right" py:if="hasattr(navigation, 'revid_list')">
@@ -34,8 +34,7 @@
                     </span>
                 	<span py:if="not navigation.prev_page_revid"> &#171; </span>
                     revision ${history.get_revno(revid)}
-                    (<span py:if="navigation.pagesize > 1">page </span>
-                    ${navigation.page_position} / ${navigation.page_count})
+                    (<span py:if="navigation.pagesize > 1">page </span>${navigation.page_position} / ${navigation.page_count})
                 	<span py:if="navigation.next_page_revid">
                     <a href="${navigation.next_page_url}" title="Next page"> &#187; </a>
                     </span>

=== modified file 'loggerhead/templates/revision.kid'
--- loggerhead/templates/revision.kid	2007-06-26 06:28:36 +0000
+++ loggerhead/templates/revision.kid	2007-09-04 17:06:03 +0000
@@ -17,24 +17,33 @@
     <span py:replace="use_collapse_buttons()"></span>
     
     <script type="text/javascript"> <!--
-    function show_sbs() {
+    function show_sbs(path) {
         collapseDisplay('style', 'sbs', 'table');
         collapseDisplay('style', 'unified', 'none');
-        document.cookie='diff=sbs';
+        document.cookie='diff=sbs; path=' + path; 
     }
-    function show_unified() {
+    function show_unified(path) {
         collapseDisplay('style', 'unified', 'table');
         collapseDisplay('style', 'sbs', 'none');
-        document.cookie='diff=unified'; 
+        document.cookie='diff=unified; path=' + path; 
     }
-    function load() {
+    function load(path) {
         sortCollapseElements();
-        if (document.cookie.indexOf('diff=unified') >= 0) { show_unified(); }
+        if (document.cookie.indexOf('diff=unified') >= 0) {
+            show_unified(path);
+        }
+        if (document.cookie.indexOf('stop=1') >= 0) {
+            return;
+        }
+        if (document.cookie.indexOf('diff=unified') >= 0) {
+            window.location.hash = window.location.hash;
+            document.cookie='stop=1; max-age=5; path=' + window.location.pathname;
+        } 
     }
     // --> </script>
 </head>
 
-<body onload="javascript:load()">
+<body onload="javascript:load('${branch.url('/')}')">
 
 ${navbar()}
 
@@ -42,32 +51,27 @@
     <span py:if="compare_revid is not None"> (compared to revision ${history.get_revno(compare_revid)}) </span>
     
     <div class="links">
-        <div> <b>&#8594;</b> <a href="${branch.url([ '/files', revid ], **util.get_context())}">
-            browse files</a> </div>
-        <div> <b>&#8594;</b> <a href="${branch.url('/changes', **util.get_context(start_revid=revid))}">
-            view branch changes</a> </div>
-        <span py:if="compare_revid is None" py:strip="True">
-            <div> <b>&#8594;</b> <a href="${branch.url([ '/bundle', revid, 'bundle.txt' ])}">
-                view/download patch</a> </div>
-        </span>
+        <div> <b>&#8594;</b> <a href="${branch.context_url([ '/files', revid ])}">
+            browse files at revision ${change.revno} </a> </div>
+        <div> <b>&#8594;</b> <a href="${branch.context_url('/changes', start_revid=revid)}">
+            view history from revision ${change.revno} </a> </div>
         <span py:if="compare_revid is not None" py:strip="True">
             <div> <b>&#8594;</b> <a href="${branch.url([ '/bundle', revid, compare_revid, 'bundle.txt' ])}">
-                view/download patch</a> </div>
+                download bundle from ${history.get_revno(compare_revid)} to ${change.revno} </a> </div>
         </span>
-        <span py:if="(remember is not None) and (compare_revid is None)" py:strip="True">
-            <div> <b>&#8594;</b> <a href="${branch.url([ '/revision', revid ],
-                **util.get_context(compare_revid=remember))}">
+        <span py:if="(remember is not None) and (compare_revid is None) and (revid != remember)" py:strip="True">
+            <div> <b>&#8594;</b> <a href="${branch.context_url([ '/revision', revid ], compare_revid=remember)}">
                 compare with revision ${history.get_revno(remember)} </a></div>
         </span>
         <span py:if="remember != revid" py:strip="True">
-            <div> <b>&#8594;</b> <a href="${branch.url([ '/revision', revid ],
-                **util.get_context(remember=revid, compare_revid=None))}">
+            <div> <b>&#8594;</b> <a href="${branch.context_url([ '/revision', revid ], remember=revid, compare_revid=None)}">
                 compare with another revision </a></div>
         </span>
         <span py:if="compare_revid is not None" py:strip="True">
-            <div> <b>&#8594;</b> <a href="${branch.url([ '/revision', revid ],
-                **util.get_context(remember=None, compare_revid=None))}">
-                stop comparing to revision ${history.get_revno(compare_revid)} </a></div>
+            <div> <b>&#8594;</b> <a href="${branch.context_url([ '/revision', compare_revid ], remember=revid, compare_revid=revid)}">
+                reverse the comparison (${change.revno} to ${history.get_revno(compare_revid)})</a></div>
+            <div> <b>&#8594;</b> <a href="${branch.context_url([ '/revision', revid ], remember=None, compare_revid=None)}">
+                stop comparing with revision ${history.get_revno(compare_revid)} </a></div>
         </span>
     </div>
 </h1>
@@ -80,7 +84,7 @@
         </tr>
         <tr>
             <th class="date">date:</th>
-            <td class="date"> ${change.date.strftime('%d %b %Y %H:%M')} </td>
+            <td class="date"> ${util.format_date(change.date)} </td>
         </tr>
 
         <tr py:if="len(change.merge_points) > 0">
@@ -142,9 +146,9 @@
     <td> ${collapse_all_button('file', 'table-row')} </td>
 
     <td class="spacey">
-        <a href="javascript:show_sbs()" class="hide-all collapse-style-sbs-show" title="collapse">
+        <a href="javascript:show_sbs('${branch.url('/')}')" class="hide-all collapse-style-sbs-show" title="collapse">
             <img src="${tg.url('/static/images/nav-small-out.gif')}" width="22" height="10" class="collapse-triangle" />show side by side</a>
-        <a href="javascript:show_unified()" class="hide-all collapse-style-unified-show" title="expand">
+        <a href="javascript:show_unified('${branch.url('/')}')" class="hide-all collapse-style-unified-show" title="expand">
             <img src="${tg.url('/static/images/nav-small-in.gif')}" width="22" height="10" class="collapse-triangle" />show unified diff</a>
     </td>
     

=== modified file 'loggerhead/tests/test_simple.py'
--- loggerhead/tests/test_simple.py	2007-09-07 18:46:20 +0000
+++ loggerhead/tests/test_simple.py	2008-02-27 02:41:36 +0000
@@ -84,7 +84,7 @@
     def test_index(self):
         testutil.create_request('/')
         link = '<a href="/project/branch">branch</a>'
-        assert link in cherrypy.response.body[0]
+        assert link in cherrypy.response.body[0].lower()
 
     def test_changes(self):
         testutil.create_request('/project/branch/changes')
@@ -128,7 +128,7 @@
     def test_index(self):
         testutil.create_request('/')
         link = '<a href="/project/branch">branch</a>'
-        assert link in cherrypy.response.body[0]
+        assert link in cherrypy.response.body[0].lower()
 
     def test_changes(self):
         testutil.create_request('/project/branch/changes')

=== modified file 'loggerhead/textindex.py'
--- loggerhead/textindex.py	2007-08-28 16:11:01 +0000
+++ loggerhead/textindex.py	2008-02-27 03:36:09 +0000
@@ -25,14 +25,11 @@
     - index: 3-letter substring -> list(revids)
 """
 
-import logging
 import os
 import re
-import threading
 import time
 
 from loggerhead import util
-from loggerhead.util import decorator
 from loggerhead.lockfile import LockFile
 from loggerhead.changecache import FakeShelf
 
@@ -65,17 +62,17 @@
     def __init__(self, history, cache_path):
         self.history = history
         self.log = history.log
-        
+
         if not os.path.exists(cache_path):
             os.mkdir(cache_path)
-        
+
         self._recorded_filename = os.path.join(cache_path, 'textindex-recorded.sql')
         self._index_filename = os.path.join(cache_path, 'textindex.sql')
-        
+
         # use a lockfile since the cache folder could be shared across different processes.
         self._lock = LockFile(os.path.join(cache_path, 'index-lock'))
         self._closed = False
-        
+
         self.log.info('Using search index; %d entries.', len(self))
 
     def _index(self):
@@ -83,10 +80,10 @@
 
     def _recorded(self):
         return FakeShelf(self._recorded_filename)
-    
+
     def _is_indexed(self, revid, recorded):
         return recorded.get(util.to_utf8(revid)) is not None
-        
+
     @with_lock
     def is_indexed(self, revid):
         recorded = self._recorded()
@@ -94,7 +91,7 @@
             return self._is_indexed(revid, recorded)
         finally:
             recorded.close()
-    
+
     @with_lock
     def __len__(self):
         recorded = self._recorded()
@@ -106,15 +103,15 @@
     @with_lock
     def close(self):
         self._closed = True
-    
+
     @with_lock
     def closed(self):
         return self._closed
-    
+
     @with_lock
     def flush(self):
         pass
-    
+
     @with_lock
     def full(self):
         recorded = self._recorded()
@@ -162,7 +159,7 @@
         finally:
             index.close(commit=True)
             recorded.close(commit=True)
-    
+
     @with_lock
     def find(self, text, revid_list=None):
         index = self._index()
@@ -170,12 +167,12 @@
             text = normalize_string(text)
             if len(text) < 3:
                 return []
-    
+
             total_set = None
             if revid_list is not None:
                 total_set = set(revid_list)
             seen_all = False
-            
+
             for i in xrange(len(text) - 2):
                 sub = text[i:i + 3]
                 revid_set = index.get(sub)
@@ -194,7 +191,7 @@
                     return []
         finally:
             index.close()
-        
+
         # tricky: if seen_all is True, one of the substring indices was ALL
         # (in other words, unindexed), so our results are actually a superset
         # of the exact answer.
@@ -204,7 +201,7 @@
         # we DON'T care, and if one of the substrings hit ALL, there's a small
         # chance that we'll give a few false positives.
         return total_set
-    
+
     def check_rebuild(self, max_time=3600):
         """
         check if there are any un-indexed revisions, and if so, index them.
@@ -241,4 +238,3 @@
             time.sleep(1)
         self.log.info('Search index completed.')
         self.flush()
-

=== modified file 'loggerhead/util.py'
--- loggerhead/util.py	2007-06-25 06:44:52 +0000
+++ loggerhead/util.py	2008-02-27 03:36:09 +0000
@@ -29,8 +29,6 @@
 import time
 import traceback
 
-import turbogears
-
 
 log = logging.getLogger("loggerhead.controllers")
 
@@ -78,6 +76,25 @@
     return year
 
 
+_g_format = '%Y-%m-%d @ %H:%M'
+
+def format_date(date):
+    if _g_format == 'fancy':
+        return fancy_format_date(date)
+    return date.strftime(_g_format)
+
+def fancy_format_date(date):
+    delta = datetime.datetime.now() - date
+    if delta.days > 300:
+        return date.strftime('%d %b %Y')
+    else:
+        return date.strftime('%d %b %H:%M')
+
+def set_date_format(format):
+    global _g_format
+    _g_format = format
+
+
 class Container (object):
     """
     Convert a dict into an object with attributes.
@@ -88,7 +105,7 @@
                 setattr(self, key, value)
         for key, value in kw.iteritems():
             setattr(self, key, value)
-    
+
     def __repr__(self):
         out = '{ '
         for key, value in self.__dict__.iteritems():
@@ -143,7 +160,7 @@
         return '%s at %s' % (username, domains[-2])
     return '%s at %s' % (username, domains[0])
 
-    
+
 def triple_factors(min_value=1):
     factors = (1, 3)
     index = 0
@@ -161,10 +178,10 @@
     """
     given a position in a maximum range, return a list of negative and positive
     jump factors for an hgweb-style triple-factor geometric scan.
-    
+
     for example, with pos=20 and max=500, the range would be:
     [ -10, -3, -1, 1, 3, 10, 30, 100, 300 ]
-    
+
     i admit this is a very strange way of jumping through revisions.  i didn't
     invent it. :)
     """
@@ -272,7 +289,7 @@
         divisor = MEG
     else:
         divisor = KILO
-    
+
     dot = size % divisor
     base = size - dot
     dot = dot * 10 // divisor
@@ -280,7 +297,7 @@
     if dot >= 10:
         base += 1
         dot -= 10
-    
+
     out = str(base)
     if (base < 100) and (dot != 0):
         out += '.%d' % (dot,)
@@ -291,7 +308,7 @@
     elif divisor == GIG:
         out += 'G'
     return out
-    
+
 
 def fill_in_navigation(navigation):
     """
@@ -305,21 +322,21 @@
     navigation.count = len(navigation.revid_list)
     navigation.page_position = navigation.position // navigation.pagesize + 1
     navigation.page_count = (len(navigation.revid_list) + (navigation.pagesize - 1)) // navigation.pagesize
-    
+
     def get_offset(offset):
         if (navigation.position + offset < 0) or (navigation.position + offset > navigation.count - 1):
             return None
         return navigation.revid_list[navigation.position + offset]
-    
+
     navigation.prev_page_revid = get_offset(-1 * navigation.pagesize)
     navigation.next_page_revid = get_offset(1 * navigation.pagesize)
-    
+
     params = { 'file_id': navigation.file_id }
     if getattr(navigation, 'query', None) is not None:
         params['q'] = navigation.query
     else:
         params['start_revid'] = navigation.start_revid
-        
+
     if navigation.prev_page_revid:
         navigation.prev_page_url = navigation.branch.url([ navigation.scan_url, navigation.prev_page_revid ], **get_context(**params))
     if navigation.next_page_revid:
@@ -432,7 +449,7 @@
     """
     return a context map that may be overriden by specific values passed in,
     but only contains keys from the list of valid context keys.
-    
+
     if 'clear' is set, only the 'remember' context value will be added, and
     all other context will be omitted.
     """

=== modified file 'start-loggerhead.py'
--- start-loggerhead.py	2007-06-19 00:09:43 +0000
+++ start-loggerhead.py	2007-09-04 17:06:03 +0000
@@ -12,7 +12,7 @@
 import cherrypy
 cherrypy.lowercase_api = True
 
-from loggerhead import daemon, release
+from loggerhead import daemon, release, util
 
 
 def make_handler(config, filename):
@@ -111,10 +111,10 @@
     Root = Root(config)
     
     # re-index every 6 hours
-    
     index_freq = config.get('cache_rebuild_frequency', 6 * 3600)
     turbogears.scheduler.add_interval_task(initialdelay=1, interval=index_freq, action=Root._check_rebuild)
-    
+
+    util.set_date_format(config.get('date_format', 'fancy'))
     try:
         turbogears.start_server(Root)
     finally:

